using System;
using System.Collections;
using System.Data;

namespace Asukaze.CassavaNET
{
	/// <summary>
	///  Cassava {@\
	/// </summary>
	public class CassavaImpl : ICassava
	{
  	// Private o
  	private DataTable table;
    private Hashtable colOptions, rowOptions;
		private RowColOption defColOpt, defRowOpt;
    private int defWidth, defHeight;
    private int fixColCount, fixRowCount;

    // RXgN^
		public CassavaImpl() {
			NewTable();
		}

 		public CassavaImpl(CassavaImpl c) {
    	table = c.table;
      colOptions = c.colOptions;
      rowOptions = c.rowOptions;
      defWidth = c.defWidth;
      defHeight = c.defHeight;
      defColOpt = c.defColOpt;
      defRowOpt = c.defRowOpt;
      fixColCount = c.fixColCount;
      fixRowCount = c.fixRowCount;
		}

		public CassavaImpl(int cols, int rows) {
			NewTable(cols, rows);
		}

    public void NewTable() {
    	NewTable(4,4);
		}

    public void NewTable(int cols, int rows) {
    	table = new DataTable();
      colOptions = new Hashtable();
      rowOptions = new Hashtable();
      defWidth = 64;
      defHeight = 24;
      defColOpt = new RowColOption(defWidth);
      defRowOpt = new RowColOption(defHeight);
      fixColCount = 0;
      fixRowCount = 0;
      for(int i=0; i<cols; i++){
        AddCol();
      }
      for(int i=0; i<rows; i++){
        AddRow();
      }
    }

    // Public vpeB
    public DataTable Table{
     	get{ return table; }
    }

    public int DefWidth {
     	get{ return defWidth; }
     	set{ defWidth = value; }
    }

    public int DefHeight {
     	get{ return defHeight; }
     	set{ defHeight = value; }
    }

    public int FixedCols {
     	get{ return fixColCount; }
     	set{ fixColCount = value; }
    }

    public int FixedRows {
     	get{ return fixRowCount; }
     	set{ fixRowCount = value; }
    }

    // Private \bh
    private Hashtable MoveRowColOptions(Hashtable hash, int from, int to){
     	Hashtable newhash = new Hashtable();
      IDictionaryEnumerator de = hash.GetEnumerator();
      while(de.MoveNext()){
        int key = (int)(de.Key);
        if(to <= key && key < from){
          newhash[key+1] = de.Value;
        }else if(key == from){
          newhash[to] = de.Value;
        }else if(from < key && key <= to){
          newhash[key-1] = de.Value;
        }else{
          newhash[key] = de.Value;
        }
      }
      return newhash;
    }

    // Public \bh
    public void MoveRow(int from, int to) {
      if(from < 0 || from >= table.Rows.Count){ return; }
      else if(to < 0){ to = 0; }
      else if(to >= table.Rows.Count){ to = table.Rows.Count - 1; }
      DataRow dr = table.Rows[from];
     	table.Rows.RemoveAt(from);
      table.Rows.InsertAt(dr, to);
      rowOptions = MoveRowColOptions(rowOptions, from, to);
    }

    public void MoveCol(int from, int to) {
      int colcount = table.Columns.Count;
      if(from < 0 || from >= table.Columns.Count){ return; }
      else if(to < 0){ to = 0; }
      else if(to >= colcount){ to = colcount - 1; }
      foreach(DataRow dr in table.Rows){
      	object[] row = dr.ItemArray;
        object val = row[from];
        if(from < to){
          for(int i=from; i<to; i++){
        	  row[i] = row[i+1];
        	}
        }else{
          for(int i=from; i>to; i--){
        	  row[i] = row[i-1];
        	}
        }
        row[to] = val;
        dr.ItemArray = row;
      }
      colOptions = MoveRowColOptions(colOptions, from, to);
    }

    // ICassava ̃Cv
  	public string GetCell(int x, int y) {
      x--; y--;
    	if(x >= 0 && x < table.Columns.Count &&
         y >= 0 && y < table.Rows.Count){
      	return table.Rows[y][x] as string;
      }else{
        return "";
      }
    }

  	public virtual bool SetCell(int x, int y, string val) {
      x--; y--;
      if(x < 0 || y < 0){
       	return false;
      }
      if(val != ""){
      	while(x >= table.Columns.Count) {
        	AddCol();
      	}
      	while(y >= table.Rows.Count) {
        	AddRow();
      	}
	     	table.Rows[y][x] = val;
      }else{
        if(x < table.Columns.Count && y < table.Rows.Count) {
          table.Rows[y][x] = val;
        }
      }
      return true;
    }

   	public int GetColCount() { return table.Columns.Count; }
		public int GetRowCount() { return table.Rows.Count; }

		public virtual void AddCol() {
    	DataColumn dc = new DataColumn("", typeof(string));
     	dc.DefaultValue = "";
     	table.Columns.Add(dc);
   	}

   	public virtual void AddRow() {
    	table.Rows.Add(table.NewRow());
   	}

   	public virtual void InsertCol(int x) {
    	x--;
     	AddCol();
     	if(x < 0){
	     	MoveCol(table.Columns.Count - 1, 0);
     	}else if(x < table.Columns.Count){
	     	MoveCol(table.Columns.Count - 1, x);
     	}
   	}

   	public virtual void InsertRow(int y) {
      if(y > table.Rows.Count){
        AddRow();
      }else{
        y--;
        if(y < 0){ y = 0; }
        table.Rows.InsertAt(table.NewRow(), y);
     	}
   	}

    public virtual void DeleteCol(int x){
      x--;
      if(x >= 0 && x < table.Columns.Count){
        table.Columns.RemoveAt(x);
      }
    }

    public virtual void DeleteRow(int y){
      y--;
      if(y >= 0 && y < table.Rows.Count){
        table.Rows.RemoveAt(y);
      }
    }

   	public int GetColWidth(int x) {
   	 	x--;
     	if(colOptions.ContainsKey(x)){
        return ((RowColOption)(colOptions[x])).Size;
      }
     	return defWidth;
   	}

    public void SetColWidth(int x, int width) {
   	 	x--;
      if(colOptions.ContainsKey(x)){
        ((RowColOption)(colOptions[x])).Size = width;
      }else{
        colOptions[x] = new RowColOption(width);
      }
    }

	 	public	int GetRowHeight(int y) {
   	 	y--;
     	if(rowOptions.ContainsKey(y)){
        return ((RowColOption)(rowOptions[y])).Size;
      }
     	return defHeight;
  	}

    public void SetRowHeight(int y, int height) {
   	 	y--;
      if(rowOptions.ContainsKey(y)){
        ((RowColOption)(rowOptions[y])).Size = height;
      }else{
        rowOptions[y] = new RowColOption(height);
      }
    }
	}
}
