using System;
using Asukaze.CassavaNET.Undo;

namespace Asukaze.CassavaNET
{
	public class CassavaU : CassavaImpl
	{
    UndoList undoList;
    UndoList seqUndoList;
    int Lock;

		public CassavaU() : base()
		{
      undoList = new UndoList();
      seqUndoList = null;
      Lock = 0;
		}

		public CassavaU(CassavaImpl b) : base(b)
		{
      undoList = new UndoList();
      Lock = 0;
		}

    public void Undo(out Range range){
      Lock++;
      undoList.Undo(out range);
      Lock--;
    }

    public void Redo(out Range range){
      Lock++;
      undoList.Redo(out range);
      Lock--;
    }

   	public override bool SetCell(int x, int y, string val) {
      if(Lock++ == 0){
        string old = GetCell(x, y);
        if(val != old){
          undoList.Add(UndoExecuter.GetSetCellDelegate(this, x, y, old));
        }
      }
      bool r = base.SetCell(x, y, val);
      Lock--;
      return r;
    }

    public override void InsertCol(int x){
      if(Lock++ == 0){
        UndoDelegate del = UndoExecuter.GetInsertColDelegate(this, x);
        Range range;
        undoList.Add(del(out range));
      }else{
        base.InsertCol(x);
      }
      Lock--;
    }

    public override void InsertRow(int y){
      if(Lock++ == 0){
        UndoDelegate del = UndoExecuter.GetInsertRowDelegate(this, y);
        Range range;
        undoList.Add(del(out range));
      }else{
        base.InsertRow(y);
      }
      Lock--;
    }

    public override void DeleteCol(int x){
      if(Lock++ == 0){
        UndoDelegate del = UndoExecuter.GetDeleteColDelegate(this, x);
        Range range;
        undoList.Add(del(out range));
      }else{
        base.DeleteCol(x);
      }
      Lock--;
    }

    public override void DeleteRow(int y){
      if(Lock++ == 0){
        UndoDelegate del = UndoExecuter.GetDeleteRowDelegate(this, y);
        Range range;
        undoList.Add(del(out range));
      }else{
        base.DeleteRow(y);
      }
      Lock--;
    }

    public void BeginUndoSequense(){
      if(Lock == 0 && seqUndoList == null){
        seqUndoList = undoList;
        undoList = new UndoList();
      }
    }

    public void EndUndoSequense(){
      if(Lock == 0 && seqUndoList != null){
        seqUndoList.Add(undoList.GetDelegate());
        undoList = seqUndoList;
        seqUndoList = null;
      }
    }

    public bool Modified{
      get{ return undoList.CanUndo; }
      set{
        if(value == false){ undoList.Clear(); }
        else if(!(undoList.CanUndo)){ undoList.Add(null); }
      }
    }
	}
}
