using System;
using System.IO;
using System.Text.RegularExpressions;

namespace Asukaze.CassavaNET.IO
{
	public class CsvWriter
	{
    private CsvWriter(){
      throw new NotImplementedException();
    }

    public static void Write(TextWriter writer, FileFormat format, ICassava data) {
     int rowCount = data.GetRowCount();
      int colCount = data.GetColCount();
      string q = format.QuoteChar.ToString();

      for(int y=1; y<=rowCount; y++){
        int right = colCount;
        if(format.OmitComma){
          while(right > 0 && data.GetCell(right,y) == ""){
            right--;
          }
        }
        for(int x=1; x<=right; x++){
          string cell = data.GetCell(x,y);

          bool quote;
          switch(format.QuoteOption){
            case Quote.None : quote = false; break;
            case Quote.Normal :
              quote = (cell.IndexOfAny(format.UsedChars) >= 0);
              break;
            case Quote.String :
              quote = !(isNumber(cell));
              break;
            case Quote.All : quote = false; break;
            default : quote = true; break;
          }
          if(quote){
            cell = q + cell.Replace(q, q + q) + q;
          }

          if(x > 1){
            writer.Write(format.SepChar);
          }
          writer.Write(cell);
        }
        writer.WriteLine();
      }
    }

    public const string strIsDigit = @"^\s*[+-]?\d*(\.\d*)?(e[+-]?\d+)?\s*$";
    private static bool isNumber(string str) {
      Regex reg = new Regex(strIsDigit, RegexOptions.Compiled);
      return reg.IsMatch(str);
    }
 	}
}
