using System;
using System.Collections;
using Asukaze.CassavaNET;

namespace Asukaze.CassavaNET.Undo
{
  public delegate UndoDelegate UndoDelegate(out Range range);

	public class UndoList
	{
    private Stack undo = new Stack();
    private Stack redo = new Stack();

		public UndoList()
		{
		}

    public void Undo(out Range range){
      range = null;
      if(undo.Count > 0){
        UndoDelegate del = undo.Pop() as UndoDelegate;
        if(del == null){ return; }
        redo.Push(del(out range));
      }
    }

    public void Redo(out Range range){
      range = null;
      if(redo.Count > 0){
        UndoDelegate del = redo.Pop() as UndoDelegate;
        if(del == null){ return; }
        undo.Push(del(out range));
      }
    }

    public void Add(UndoDelegate del){
      undo.Push(del);
      redo.Clear();
    }

    public void Clear(){
      undo.Clear();
      redo.Clear();
    }

    public bool CanUndo{ get{ return (undo.Count > 0); } }
    public bool CanRedo{ get{ return (redo.Count > 0); } }


    public UndoDelegate UndoAll(out Range range){
      range = null;
      UndoList ul = new UndoList();
      while(undo.Count > 0){
        UndoDelegate del = undo.Pop() as UndoDelegate;
        if(del != null){ ul.Add(del(out range)); }
      }
      return ul.GetDelegate();
    }

    public UndoDelegate GetDelegate(){
      if(undo.Count == 1){
        return undo.Peek() as UndoDelegate;
      }else{
        return new UndoDelegate(UndoAll);
      }
    }
	}

  public class UndoExecuter{
    private ICassava data;
    private int x,y;
    private string str;
    private int colcount, rowcount;

    private UndoExecuter(ICassava d, int x, int y, string s){
      this.data = d;
      this.x = x;
      this.y = y;
      this.str = s;
      this.colcount = data.GetColCount();
      this.rowcount = data.GetRowCount();
    }

    private UndoDelegate SetCell(out Range range){
      string v = data.GetCell(x, y);
      data.SetCell(x, y, str);
      UndoDelegate del = GetSetCellDelegate(data,x,y,v);
      while(data.GetColCount() > colcount){
        data.DeleteCol(data.GetColCount());
      }
      while(data.GetRowCount() > rowcount){
        data.DeleteRow(data.GetRowCount());
      }
      range = new Range(x,y,x,y);
      return del;
    }

    public static UndoDelegate GetSetCellDelegate(ICassava data, int x, int y, string val){
      UndoExecuter ue = new UndoExecuter(data, x, y, val);
      return new UndoDelegate(ue.SetCell);
    }

    private UndoDelegate InsertCol(out Range range){
      data.InsertCol(x);
      range = new Range(x,1,x,data.GetRowCount());
      return GetDeleteColDelegate(data, x);
    }

    public static UndoDelegate GetInsertColDelegate(ICassava data, int x){
      UndoExecuter ue = new UndoExecuter(data, x, 0, null);
      return new UndoDelegate(ue.InsertCol);
    }

    private UndoDelegate DeleteCol(out Range range){
      UndoList ul = new UndoList();
      for(int i=1; i<=data.GetRowCount(); i++){
        if(data.GetCell(x, i) != ""){
          ul.Add(GetSetCellDelegate(data, x, i, data.GetCell(x, i)));
        }
      }
      data.DeleteCol(x);
      ul.Add(GetInsertColDelegate(data, x));
      range = new Range(x,1,x,data.GetRowCount());
      return ul.GetDelegate();
    }

    public static UndoDelegate GetDeleteColDelegate(ICassava data, int x){
      UndoExecuter ue = new UndoExecuter(data, x, 0, null);
      return new UndoDelegate(ue.DeleteCol);
    }

    private UndoDelegate InsertRow(out Range range){
      data.InsertRow(y);
      range = new Range(1,y,data.GetColCount(),y);
      return GetDeleteRowDelegate(data, y);
    }

    public static UndoDelegate GetInsertRowDelegate(ICassava data, int y){
      UndoExecuter ue = new UndoExecuter(data, 0, y, null);
      return new UndoDelegate(ue.InsertRow);
    }

    private UndoDelegate DeleteRow(out Range range){
      UndoList ul = new UndoList();
      for(int i=1; i<=data.GetColCount(); i++){
        if(data.GetCell(i, y) != ""){
          ul.Add(GetSetCellDelegate(data, i, y, data.GetCell(i, y)));
        }
      }
      data.DeleteRow(y);
      ul.Add(GetInsertRowDelegate(data, y));
      range = new Range(1,y,data.GetColCount(),y);
      return ul.GetDelegate();
    }

    public static UndoDelegate GetDeleteRowDelegate(ICassava data, int y){
      UndoExecuter ue = new UndoExecuter(data, 0, y, null);
      return new UndoDelegate(ue.DeleteRow);
    }

  }
}
